//---------------------------------------------------------------------------
#ifndef ObjGrafH
#define ObjGrafH
//---------------------------------------------------------------------------

// Clases de Excepciones

class EFueraRango {};

// Clase TObjGraf
class TObjGraf {
private:
  int FX;
  int FY;
  TColor FColor;

  void        SetX     (int _X) throw (EFueraRango);
  void        SetY     (int _Y) throw (EFueraRango);
  virtual int GetAncho (void) = 0; // Funciones virtuales puras
  virtual int GetAlto  (void) = 0; //(deben definirse en los descendientes)

protected:
  TPaintBox *PaintBox;

public:
  TObjGraf (TPaintBox *_PaintBox, TColor _Color=clBlack, int _X=0, int _Y=0);
  TObjGraf (TObjGraf *ObjGraf);

// Funciones virtuales puras(deben definirse en los descendientes)
  virtual void Mostrar (void) = 0;

  __property int    X     = {read=FX,     write=SetX};
  __property int    Y     = {read=FY,     write=SetY};
  __property TColor Color = {read=FColor, write=FColor};
  __property int    Ancho = {read=GetAncho};
  __property int    Alto  = {read=GetAlto};
};

// Clase TCirculo
class TCirculo : public TObjGraf {
private:
  int FRadio;

  inline int GetAncho (void) {return FRadio*2;};
  inline int GetAlto  (void) {return FRadio*2;};

public:
  TCirculo (TPaintBox *_PaintBox, TColor _Color=clBlack,
    int _X=0, int _Y=0, int _Radio=1);

  virtual void Mostrar (void);

  __property int Radio = {read=FRadio, write=FRadio};
};


// Tipo definido para la direccin de TPelota
enum TDireccion {S = 1, N = 2, E = 4, O = 8, SE = 5, NE = 6, SO = 9, NO = 10};

// Clase TPelota
class TPelota : public TCirculo {
private:
  int FDirX;
  int FDirY;
  int FVelocidad;

  void       SetDireccion (TDireccion _Direccion);
  TDireccion GetDireccion (void);

public:
  TPelota (TPaintBox *_PaintBox, TColor _Color=clBlack, int _X=0,
    int _Y=0, int _Radio=1, TDireccion _Direccion=SE, int _Velocidad=5);

  void Mostrar(void);
  void Borrar (void);
  void __fastcall Mover  (void);

  __property int        Velocidad = {read=FVelocidad,   write=FVelocidad};
  __property TDireccion Direccion = {read=GetDireccion, write=SetDireccion};

  /* La codificacin de Direccin es la siguiente:
       NO   N    NE
        10  2   6
         \  |  /
          \ | /
   O 8 ----   ---- 4 E
          / | \
         /  |  \
        9   1   5
      SO    S    SE */

};


// Clase TCuadrado
class TCuadrado : public TObjGraf {
private:
  int FLado;
  inline int GetAncho (void) {return FLado;};
  inline int GetAlto  (void) {return FLado;};

public:
  TCuadrado (TPaintBox *_PaintBox, TColor _Color=clBlack,
    int _X=0, int _Y=0, int _Lado=1);

  virtual void Mostrar (void);

  __property int Lado = {read=FLado, write=FLado};
};
#endif
